"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const colors_1 = require("colors");
const sh = require("shelljs");
const node_fs = require("fs");
const cluster = require("cluster");
const child_process_1 = require("child_process");
/**
 * This is a CLI tool helps you do execute cmds per given folder. If you try to do the same
 * with a normal cd command, it will do it in the wrong folder. So this tool simply
 * navigates to the proper folders and does `your command`. That's it. #ez #boring
 */
function eisd(commandToExecute, directoriesToUse, allowFailures) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!commandToExecute) {
            process.stdout.write(colors_1.red("ERROR: No command given..\n"));
            throw new Error("No command found.");
        }
        if (!directoriesToUse.length) {
            process.stdout.write(colors_1.red("ERROR: No directories given..\n"));
            throw new Error("No directories found.");
        }
        const { directoriesSucces, directoriesFailed } = yield startMaster(commandToExecute, directoriesToUse, allowFailures);
        if (directoriesSucces.length) {
            process.stdout.write("\n\n");
            process.stdout.write(colors_1.green(`Done executing '${commandToExecute}' in the following directories:`));
            process.stdout.write("\n");
            process.stdout.write(colors_1.green(directoriesSucces.join(", ")));
            process.stdout.write("\n");
        }
        if (directoriesFailed.length) {
            process.stdout.write("\n\n");
            process.stdout.write(colors_1.red(`Failed executing '${commandToExecute}' in the following directories:`));
            process.stdout.write("\n");
            process.stdout.write(colors_1.green(directoriesFailed.join(", ")));
            process.stdout.write("\n");
        }
    });
}
exports.eisd = eisd;
function startMaster(commandToExecute, directoriesToUse, allowFailures) {
    return new Promise(resolve => {
        const directoriesSucces = [];
        const directoriesFailed = [];
        cluster.fork({ commandToExecute, directory: directoriesToUse.shift() });
        cluster.on("message", (worker, message) => {
            worker.kill();
            message.result === "SUCCESS"
                ? directoriesSucces.push(message.directory)
                : directoriesFailed.push(message.directory);
            // No more directories or we there is a failure and we don't allow any? Resolve!
            if (!directoriesToUse.length || (!allowFailures && directoriesFailed.length)) {
                resolve({ directoriesSucces, directoriesFailed });
            }
            else {
                process.stdout.write("\n\n");
                cluster.fork({ commandToExecute, directory: directoriesToUse.shift() });
            }
        });
    });
}
exports.startMaster = startMaster;
function executeWork(commandToExecute, directory) {
    const result = { directory, result: "SUCCESS" };
    try {
        node_fs.accessSync(directory, node_fs.constants.F_OK & node_fs.constants.R_OK);
    }
    catch (e) {
        process.stdout.write(colors_1.red(`ERROR: Could not access path "${directory}"`));
        result.result = "ERROR";
        process.send(result);
        process.exit(1);
    }
    sh.cd(directory);
    console.log(colors_1.yellow(`Executing command "${commandToExecute}" in: ${directory}`));
    const child = child_process_1.exec(commandToExecute);
    child.stdout.on("data", text => {
        process.stdout.write(text);
    });
    child.stderr.on("data", error => {
        result.result = "ERROR";
        console.error(colors_1.red(`Error when executing "${commandToExecute}" in: ${directory}`));
        process.stdout.write(error);
    });
    // Use stderr.on("end") since it's always being called last, which is not the case for stdout.on("end")
    child.stderr.on("end", () => {
        result.result === "SUCCESS"
            ? process.stdout.write(colors_1.green("Done"))
            : process.stdout.write(colors_1.red("Done with errors"));
        process.send(result);
    });
}
if (cluster.isWorker) {
    executeWork(process.env.commandToExecute, process.env.directory);
}
